/**
* File: War3Source_arsenal.sp
* Description: The Arsenal race for War3Source.
* Author(s): SenatoR
*/

#pragma semicolon 1

#include <sourcemod>
#include "W3SIncs/War3Source_Interface"

new thisRaceID;

new SKILL_GUNS,SKILL_CRITICAL,SKILL_BOUNTY,SKILL_CHWEAPON;
new Handle:ultCooldownCvar;

// Skill Data Arrays
new Float:CritChance[7]={0.0,0.03,0.06,0.09,0.12,0.15,0.20};
new BountyCash[7]={0,50,100,200,400,800,1600};

public Plugin:myinfo = 
{
	name = "War3Source Race - Arsenal",
	author = "SenatoR",
	description = "The Arsenal race for War3Source.",
	version = "1.0",
	url = "none"
};

public OnPluginStart()
{
	ultCooldownCvar=CreateConVar("war3_pistoleer_ult_cooldown","5.0","Cooldown for Exclusive Suppliers");
}

public OnWar3LoadRaceOrItemOrdered2(num)
{
	if (num==100)
	{
		thisRaceID=War3_CreateNewRace("Оружейник","arsenal");
		SKILL_GUNS=War3_AddRaceSkill(thisRaceID,"Модернизация","Усиление урона",false,5);
		SKILL_CRITICAL=War3_AddRaceSkill(thisRaceID,"Игольчатые пули ","Шанс нанести большой урон",false,6);
		SKILL_BOUNTY=War3_AddRaceSkill(thisRaceID,"Мародер","Получаете больше денег за убийство",false,6);
		SKILL_CHWEAPON=War3_AddRaceSkill(thisRaceID,"Арсенал","Выбор нового оружия",true,1); 
		War3_CreateRaceEnd(thisRaceID);
	}
}

public OnWar3EventSpawn(client)
{
	new race=War3_GetRace(client);
	if(race==thisRaceID)
	{
		War3_WeaponRestrictTo(client,thisRaceID,"weapon_knife",1);
		DoPistolMenu(client);
	}
}

// Opens pistol menu
public DoPistolMenu(client)
{
	new Handle:pistolMenu=CreateMenu(War3Source_PistolMenu_Selected);
	SetMenuExitButton(pistolMenu,false);
	SetMenuPagination(pistolMenu,MENU_NO_PAGINATION);
	SetMenuTitle(pistolMenu,"== Weapon Menu ==");
	
	new gunlevel = War3_GetSkillLevel(client,thisRaceID,SKILL_GUNS);
	
	AddMenuItem(pistolMenu,"weapon_famas","Famas");
	AddMenuItem(pistolMenu,"weapon_galil","Galil",(gunlevel>0)?ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);
	AddMenuItem(pistolMenu,"weapon_aug","AUG",(gunlevel>1)?ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);
	AddMenuItem(pistolMenu,"weapon_sg552","SG552",(gunlevel>2)?ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);
	AddMenuItem(pistolMenu,"weapon_ak47","AK47",(gunlevel>3)?ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);
	AddMenuItem(pistolMenu,"weapon_m4a1","M4A1",(gunlevel>4)?ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);
	DisplayMenu(pistolMenu,client,MENU_TIME_FOREVER);
}

// Pistol menu functionality
public War3Source_PistolMenu_Selected(Handle:menu,MenuAction:action,client,selection)
{
	if(action==MenuAction_Select)
	{
		if(ValidPlayer(client) && War3_GetRace(client)==thisRaceID)
		{
			decl String:newRestrict[64];
			decl String:weaponName[32];
			decl String:SelectionDispText[256];
			new SelectionStyle;
			
			GetMenuItem(menu,selection,weaponName,sizeof(weaponName),SelectionStyle,SelectionDispText,sizeof(SelectionDispText));
			Format(newRestrict,64,"weapon_knife,%s",weaponName);
			
			War3_WeaponRestrictTo(client,thisRaceID,newRestrict,2);
			GivePlayerItem(client,weaponName);
		}
	}
	if(action==MenuAction_Cancel)
	{
		if(ValidPlayer(client) && War3_GetRace(client)==thisRaceID)
		{
			War3_WeaponRestrictTo(client,thisRaceID,"weapon_knife,weapon_famas",2);
			GivePlayerItem(client,"weapon_famas");
		}
	}
	if(action==MenuAction_End)
	{
		CloseHandle(menu);
	}
}

// Do bounty collection on player death
public OnWar3EventDeath(victim,attacker)
{
	new race=War3_GetRace(attacker);
	new vteam=GetClientTeam(victim);
	new ateam=GetClientTeam(attacker);
	if (victim>0 && attacker>0 && victim!=attacker && race==thisRaceID && !Hexed(attacker) && vteam!=ateam)
	{
		new level = War3_GetSkillLevel(attacker,thisRaceID,SKILL_BOUNTY);
		if (level)
		{
			new money=GetCSMoney(attacker);
			money += BountyCash[level];
			SetCSMoney(attacker,money);
			W3FlashScreen(attacker,RGBA_COLOR_GREEN);
			War3_ChatMessage(attacker,"Collected a $%d bounty",BountyCash[level]);
		}
	}
}

// Do critical hit calculation for lucky shot
public OnW3TakeDmgBullet(victim,attacker,Float:damage)
{
	if(IS_PLAYER(victim)&&IS_PLAYER(attacker)&&victim>0&&attacker>0&&attacker!=victim)
	{
		// Not going to allow critical hit on team attacks for now
		new vteam=GetClientTeam(victim);
		new ateam=GetClientTeam(attacker);
		if(vteam!=ateam)
		{
			new race_attacker=War3_GetRace(attacker);
			new skill_attacker=War3_GetSkillLevel(attacker,race_attacker,SKILL_CRITICAL);
			if(race_attacker==thisRaceID && skill_attacker>0 && !Hexed(attacker,false))
			{
				// Will not factor in skill immunity
				if (GetRandomFloat(0.0,1.0)<=CritChance[skill_attacker])
				{
					if (War3_DealDamage(victim,30,attacker,DMG_BULLET,"luckyshot",W3DMGORIGIN_SKILL,W3DMGTYPE_PHYSICAL))
					{
						W3PrintSkillDmgHintConsole( victim, attacker, War3_GetWar3DamageDealt(),SKILL_CRITICAL );
						W3FlashScreen(victim,RGBA_COLOR_RED);
					}
				}
			}
		}
	}
}



public OnRaceChanged( client,oldrace,newrace )
{
	if(newrace!=thisRaceID)
	{
		War3_WeaponRestrictTo(client,thisRaceID,"");
	}
	else
	{
		if (IsPlayerAlive(client))
		{
			War3_WeaponRestrictTo(client,thisRaceID,"weapon_knife",1);
			DoPistolMenu(client);
		}
	}
}

public OnUltimateCommand(client,race,bool:pressed)
{
	if(race==thisRaceID && pressed && IsPlayerAlive(client))
	{
		new skill=War3_GetSkillLevel(client,race,SKILL_CHWEAPON);
		if(skill&&War3_SkillNotInCooldown(client,thisRaceID,SKILL_CHWEAPON,true))
		{
			DoPistolMenu(client);
			new Float:cooldown=GetConVarFloat(ultCooldownCvar);
			War3_CooldownMGR(client,cooldown,thisRaceID,SKILL_CHWEAPON );
		}
	}
}